Package: sRACIPE
Type: Package
Title: Systems biology tool to simulate gene regulatory circuits
Version: 2.2.0
Authors@R: c(
    person("Mingyang", "Lu", , "m.lu@northeastern.edu", role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0001-8158-0593")
    ),
    person("Vivek", "Kohar", role = "aut"
    ),
    person("Aidan", "Tillman", role = "aut"
    ),
    person("Daniel", "Ramirez", role = "aut")
    )
Description: sRACIPE implements a randomization-based method for gene circuit modeling. It allows us to study the effect of both the gene expression noise and the parametric variation on any gene regulatory circuit (GRC) using only its topology, and simulates an ensemble of models with random kinetic parameters at multiple noise levels. Statistical analysis of the generated gene expressions reveals the basin of attraction and stability of various phenotypic states and their changes associated with intrinsic and extrinsic noises. sRACIPE provides a holistic picture to evaluate the effects of both the stochastic nature of cellular processes and the parametric variation. 
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: knitr, BiocStyle, rmarkdown, tinytest
VignetteBuilder: knitr
LinkingTo: Rcpp
biocViews: ResearchField, SystemsBiology, MathematicalBiology,
        GeneExpression, GeneRegulation, GeneTarget
Depends: R (>= 3.6.0),SummarizedExperiment, methods, Rcpp
Imports: ggplot2, reshape2, MASS, RColorBrewer, gridExtra,visNetwork,
        gplots, umap, htmlwidgets, S4Vectors, BiocGenerics, grDevices,
        stats, utils, graphics, doFuture, doRNG, future, foreach
RoxygenNote: 7.3.2
URL: https://github.com/lusystemsbio/sRACIPE, https://geneex.jax.org/,
        https://vivekkohar.github.io/sRACIPE/
BugReport: https://github.com/lusystemsbio/sRACIPE/issues
LazyData: true
git_url: https://git.bioconductor.org/packages/sRACIPE
git_branch: RELEASE_3_22
git_last_commit: 26d0307
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 06:47:04 UTC; biocbuild
Author: Mingyang Lu [aut, cre] (ORCID: <https://orcid.org/0000-0001-8158-0593>),
  Vivek Kohar [aut],
  Aidan Tillman [aut],
  Daniel Ramirez [aut]
Maintainer: Mingyang Lu <m.lu@northeastern.edu>
