# Generated by roxygen2: do not edit by hand

export("sracipeCircuit<-")
export("sracipeConfig<-")
export("sracipeIC<-")
export("sracipeParams<-")
export(.RacipeSE)
export(RacipeSE)
export(densityPlot)
export(sracipeCircuit)
export(sracipeCombineRacipeSE)
export(sracipeConfig)
export(sracipeConverge)
export(sracipeConvergeDist)
export(sracipeGenParamNames)
export(sracipeGetTS)
export(sracipeHeatmapSimilarity)
export(sracipeIC)
export(sracipeKnockDown)
export(sracipeNormalize)
export(sracipeOverExp)
export(sracipeParams)
export(sracipePlotCircuit)
export(sracipePlotData)
export(sracipePlotParamBifur)
export(sracipeSimulate)
export(sracipeUniqueStates)
exportClasses(RacipeSE)
exportMethods("annotation<-")
exportMethods(annotation)
exportMethods(sracipeCircuit)
exportMethods(sracipeCombineRacipeSE)
exportMethods(sracipeConvergeDist)
exportMethods(sracipeGetTS)
exportMethods(sracipeKnockDown)
exportMethods(sracipeUniqueStates)
import(MASS)
import(RColorBrewer)
import(Rcpp)
import(SummarizedExperiment)
import(doFuture)
import(doRNG)
import(foreach)
import(future)
import(ggplot2)
import(gplots)
import(grDevices)
import(gridExtra)
import(htmlwidgets)
import(methods)
import(reshape2)
import(umap)
import(visNetwork)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,metadata)
importFrom(gplots,heatmap.2)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,par)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,prcomp)
importFrom(stats,sd)
importFrom(utils,data)
importFrom(utils,read.table)
importFrom(utils,write.table)
importMethodsFrom(BiocGenerics,"annotation<-")
importMethodsFrom(BiocGenerics,annotation)
useDynLib(sRACIPE)
