# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

generateThresholds <- function(geneInteraction, thresholdGene, config) {
    .Call('_sRACIPE_generateThresholds', PACKAGE = 'sRACIPE', geneInteraction, thresholdGene, config)
}

limitcyclesGRC <- function(geneInteraction, outFileLC, outFileLCIC, config, modelConverg, inFileParams, inFileGE, geneTypes, stepper = 1L) {
    .Call('_sRACIPE_limitcyclesGRC', PACKAGE = 'sRACIPE', geneInteraction, outFileLC, outFileLCIC, config, modelConverg, inFileParams, inFileGE, geneTypes, stepper)
}

simulateGRCCpp <- function(geneInteraction, config, outFileGE, outFileParams, outFileIC, outFileConverge, geneTypes, signalVals, signalingTypes, stepper = 1L) {
    .Call('_sRACIPE_simulateGRCCpp', PACKAGE = 'sRACIPE', geneInteraction, config, outFileGE, outFileParams, outFileIC, outFileConverge, geneTypes, signalVals, signalingTypes, stepper)
}

